# Generated by Django 5.2.8 on 2025-12-02 12:59

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='UService',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('title_translate_key', models.CharField(max_length=255)),
                ('logo', models.FileField(upload_to='media/logo')),
                ('type', models.CharField(choices=[('FILE_UPLOAD', 'File Upload'), ('API_INTEGRATION', 'Api Integration'), ('ADMIN', 'Admin')], max_length=20)),
                ('ip', models.GenericIPAddressField()),
                ('port', models.PositiveSmallIntegerField()),
            ],
            options={
                'ordering': ['name'],
            },
        ),
    ]
